/*
 * OpenClinica is distributed under the
 * GNU Lesser General Public License (GNU LGPL).

 * For details see: http://www.openclinica.org/license
 * copyright 2003-2005 Akaza Research
 */
package org.akaza.openclinica.bean.core;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

/**
 * @author Jun Xu
 *  
 */
public class DiscrepancyNoteType extends Term {
  public static final DiscrepancyNoteType INVALID = new DiscrepancyNoteType(0, "invalid", null);

  public static final DiscrepancyNoteType FAILEDVAL =
    new DiscrepancyNoteType(1, "Failed Validation Check",null);

  public static final DiscrepancyNoteType INCOMPLETE = new DiscrepancyNoteType(2, "Incomplete", null);

  public static final DiscrepancyNoteType UNCLEAR = 
    new DiscrepancyNoteType(3, "Unclear/Unreadable", null);

  public static final DiscrepancyNoteType ANNOTATION =
    new DiscrepancyNoteType(4, "Annotation",null);

  public static final DiscrepancyNoteType OTHER = new DiscrepancyNoteType(5, "Other",null);

  private static final DiscrepancyNoteType[] members = 
  {FAILEDVAL, INCOMPLETE, UNCLEAR, ANNOTATION,OTHER};

  public static final List list = Arrays.asList(members);

  private List privileges;

  private DiscrepancyNoteType(int id, String name, Privilege[] myPrivs) {
    super(id, name);
    
  }

  private DiscrepancyNoteType() {
  }

  public static boolean contains(int id) {
    return Term.contains(id, list);
  }

  public static DiscrepancyNoteType get(int id) {
    return (DiscrepancyNoteType) Term.get(id, list);
  }

  public static DiscrepancyNoteType getByName(String name) {
    for (int i = 0; i < list.size(); i++) {
      DiscrepancyNoteType temp = (DiscrepancyNoteType) list.get(i);
      if (temp.getName().equals(name)) {
        return temp;
      }
    }
    return INVALID;
  }

  public static ArrayList toArrayList() {
    return new ArrayList(list);
  }

  public boolean hasPrivilege(Privilege p) {
    Iterator it = privileges.iterator();

    while (it.hasNext()) {
      Privilege myPriv = (Privilege) it.next();
      if (myPriv.equals(p)) {
        return true;
      }
    }
    return false;
  }

}